use cgmath::{vec3, Matrix3, Vector3};
use engine::prelude::{
    cgmath::{Deg, Vector2},
    *,
};

pub struct Objects;

impl Objects {
    pub fn create_cuboid(offset: Vector3<f32>, extent: Vector3<f32>) -> [PositionNormal; 36] {
        [
            // near
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z + extent.z),
                vec3(0.0, -1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(
                    offset.x + extent.x,
                    offset.y + extent.y,
                    offset.z + extent.z,
                ),
                vec3(0.0, -1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z),
                vec3(0.0, -1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z),
                vec3(0.0, -1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z),
                vec3(0.0, -1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z + extent.z),
                vec3(0.0, -1.0, 0.0),
            ),
            // right
            PositionNormal::new(
                vec3(
                    offset.x + extent.x,
                    offset.y + extent.y,
                    offset.z + extent.z,
                ),
                vec3(1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z + extent.z),
                vec3(1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z),
                vec3(1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z),
                vec3(1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y + extent.y, offset.z),
                vec3(1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(
                    offset.x + extent.x,
                    offset.y + extent.y,
                    offset.z + extent.z,
                ),
                vec3(1.0, 0.0, 0.0),
            ),
            // far
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z + extent.z),
                vec3(0.0, 1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y, offset.z + extent.z),
                vec3(0.0, 1.0, 0.0),
            ),
            PositionNormal::new(vec3(offset.x, offset.y, offset.z), vec3(0.0, 1.0, 0.0)),
            PositionNormal::new(vec3(offset.x, offset.y, offset.z), vec3(0.0, 1.0, 0.0)),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z),
                vec3(0.0, 1.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z + extent.z),
                vec3(0.0, 1.0, 0.0),
            ),
            // left
            PositionNormal::new(
                vec3(offset.x, offset.y, offset.z + extent.z),
                vec3(-1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z + extent.z),
                vec3(-1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z),
                vec3(-1.0, 0.0, 0.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z),
                vec3(-1.0, 0.0, 0.0),
            ),
            PositionNormal::new(vec3(offset.x, offset.y, offset.z), vec3(-1.0, 0.0, 0.0)),
            PositionNormal::new(
                vec3(offset.x, offset.y, offset.z + extent.z),
                vec3(-1.0, 0.0, 0.0),
            ),
            // top
            PositionNormal::new(
                vec3(offset.x, offset.y, offset.z + extent.z),
                vec3(0.0, 0.0, 1.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z + extent.z),
                vec3(0.0, 0.0, 1.0),
            ),
            PositionNormal::new(
                vec3(
                    offset.x + extent.x,
                    offset.y + extent.y,
                    offset.z + extent.z,
                ),
                vec3(0.0, 0.0, 1.0),
            ),
            PositionNormal::new(
                vec3(
                    offset.x + extent.x,
                    offset.y + extent.y,
                    offset.z + extent.z,
                ),
                vec3(0.0, 0.0, 1.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z + extent.z),
                vec3(0.0, 0.0, 1.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y, offset.z + extent.z),
                vec3(0.0, 0.0, 1.0),
            ),
            // bottom
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y + extent.y, offset.z),
                vec3(0.0, 0.0, -1.0),
            ),
            PositionNormal::new(
                vec3(offset.x, offset.y + extent.y, offset.z),
                vec3(0.0, 0.0, -1.0),
            ),
            PositionNormal::new(vec3(offset.x, offset.y, offset.z), vec3(0.0, 0.0, -1.0)),
            PositionNormal::new(vec3(offset.x, offset.y, offset.z), vec3(0.0, 0.0, -1.0)),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y, offset.z),
                vec3(0.0, 0.0, -1.0),
            ),
            PositionNormal::new(
                vec3(offset.x + extent.x, offset.y + extent.y, offset.z),
                vec3(0.0, 0.0, -1.0),
            ),
        ]
    }

    pub fn create_flat_quad(corners: [Vector2<f32>; 4]) -> [PositionNormal; 6] {
        let up = vec3(0.0, 0.0, 1.0);

        [
            PositionNormal::new(corners[0].extend(0.0), up),
            PositionNormal::new(corners[1].extend(0.0), up),
            PositionNormal::new(corners[2].extend(0.0), up),
            PositionNormal::new(corners[2].extend(0.0), up),
            PositionNormal::new(corners[3].extend(0.0), up),
            PositionNormal::new(corners[0].extend(0.0), up),
        ]
    }

    pub fn create_cylinder(radius: f32, height: f32, detail: u8) -> Vec<PositionNormal> {
        let center = vec3(0.0, 0.0, 0.0);
        let reference_point = vec3(radius, 0.0, 0.0);

        let split_degree = Deg(360.0 / detail as f32);

        let mut vertices = Vec::with_capacity(detail as usize * 12);

        for i in 0..detail {
            let first_outer = Matrix3::from_angle_z(split_degree * i as f32) * reference_point;
            let second_outer =
                Matrix3::from_angle_z(split_degree * (i + 1) as f32) * reference_point;

            // bottom triangle
            vertices.push(PositionNormal::new(center, vec3(0.0, 0.0, -1.0)));
            vertices.push(PositionNormal::new(first_outer, vec3(0.0, 0.0, -1.0)));
            vertices.push(PositionNormal::new(second_outer, vec3(0.0, 0.0, -1.0)));

            // top triangle
            vertices.push(PositionNormal::new(
                center + vec3(0.0, 0.0, height),
                vec3(0.0, 0.0, 1.0),
            ));
            vertices.push(PositionNormal::new(
                first_outer + vec3(0.0, 0.0, height),
                vec3(0.0, 0.0, 1.0),
            ));
            vertices.push(PositionNormal::new(
                second_outer + vec3(0.0, 0.0, height),
                vec3(0.0, 0.0, 1.0),
            ));

            // outside face
            vertices.push(PositionNormal::new(
                first_outer + vec3(0.0, 0.0, height),
                first_outer.normalize(),
            ));
            vertices.push(PositionNormal::new(first_outer, first_outer.normalize()));
            vertices.push(PositionNormal::new(second_outer, second_outer.normalize()));
            vertices.push(PositionNormal::new(second_outer, second_outer.normalize()));
            vertices.push(PositionNormal::new(
                second_outer + vec3(0.0, 0.0, height),
                second_outer.normalize(),
            ));
            vertices.push(PositionNormal::new(
                first_outer + vec3(0.0, 0.0, height),
                first_outer.normalize(),
            ));
        }

        vertices
    }
}
